# IssueTrackerMassContainer
#
# Peter Bengtsson <mail@peterbe.com>
# License: ZPL
#

__doc__="""IssueTrackerMassContainer is a folderish container
for Zope where you have multiple instances of the IssueTrackerProduct
By Peter Bengtsson <mail@peterbe.com>

Credits:
"""
__version__='0.0.3'


# python
import os

# zope
from Products.PageTemplates.PageTemplateFile import PageTemplateFile as PTF
from Globals import Persistent, InitializeClass, package_home, DTMLFile
from Products.PythonScripts.standard import html_quote, url_quote_plus
from OFS import Folder
from DocumentTemplate import sequence
from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from Products.PythonScripts.PythonScript import PythonScript

# product
from MassContainerConstants import *
import MassContainerUtils
from MassContainerUtils import cookIdAndTitle

try:
    from Products.IssueTrackerUtils import _replace_special_chars
except:
    _replace_special_chars = None

manage_addMassContainerForm = PTF('zpt/addMassContainerForm', globals())


def manage_addMassContainer(dispatcher, id, title='',
                           use_native_header=0,
                           REQUEST=None):
    """ add IssueTrackerMassContainer instance via the web """
    dest = dispatcher.Destination()
    masscontainer = MassContainer(id, title, use_native_header)
    dest._setObject(id, masscontainer)
    self = dest._getOb(id)
    
    self.DeployMassStandards()

    if REQUEST is not None:
        # whereto next?
        if REQUEST.has_key('addandedit'):
            REQUEST.RESPONSE.redirect(self.absolute_url()+'/manage_workspace')
        elif REQUEST.has_key('DestinationURL'):
            REQUEST.RESPONSE.redirect(REQUEST.DestinationURL+'/manage_workspace')
        else:
            REQUEST.RESPONSE.redirect(REQUEST.URL1+'/manage_workspace')

class MassContainer(Folder.Folder, Persistent):
    """ MassContainer class """
    
    meta_type = MASSCONTAINER_METATYPE

    _properties=({'id':'title',            'type': 'string', 'mode':'w'},
                 {'id':'use_native_header','type': 'boolean','mode':'w'}
                 )
    
    security=ClassSecurityInfo()
    
    manage_options = Folder.Folder.manage_options[:3] + \
        ({'label':'Deploy Standards', 'action':'manage_DeployStandards'},) \
        + Folder.Folder.manage_options[3:]    

    def __init__(self, id, title='', use_native_header=0):
        """ Init MassContainer class """
        self.id = str(id)
        self.title = str(title)
        self.use_native_header = use_native_header
        
       
    def getRoot(self):
        """ return self class """
        return self
    
    def getRootURL(self):
        """ return self's absolute_url() """
        return self.getRoot().absolute_url()

    def manage_DeployStandards(self):
        """ tab entry to DeployMassStandards """
        durl = self.getRootURL()+'/manage_workspace'
        self.DeployMassStandards(remove_oldstuff=1, DestinationURL=durl)
        
        
    def DeployMassStandards(self, remove_oldstuff=0, DestinationURL=None):
        """ copy images and other documents into the instance unless they
            are already there
        """
        t={}
        
        # create folders
        root = self.getRoot()

        osj = os.path.join
        standards_home = osj(package_home(globals()),'standards')
        t = self._deployImages(root, standards_home,
                               t=t, remove_oldstuff=remove_oldstuff)

        #www_home = osj(standards_home,'www')
        #t = self._deployImages(root.www, www_home,
                               #t=t, remove_oldstuff=remove_oldstuff)

        # shortcut
        addPG = root.manage_addProduct['PageTemplates'].manage_addPageTemplate
        AddParam2URL = MassContainerUtils.AddParam2URL
        
        for filestr in os.listdir(standards_home):
            if filestr[-5:] == '.dtml':
                id, title = cookIdAndTitle(filestr.replace('.dtml',''))
                if hasattr(root, id) and remove_oldstuff:
                    root.manage_delObjects([id])
                
                if not hasattr(root, id):
                    file = DTMLFile('standards/%s'%filestr.replace('.dtml',''), globals()).read()
                    root.manage_addDTMLDocument(id, title, file=file)
                    t[id] ="DTML Document"
            elif filestr[-4:] == '.zpt':
                id, title = cookIdAndTitle(filestr.replace('.zpt',''))
                if hasattr(root, id) and remove_oldstuff:
                    root.manage_delObjects([id])
                    
                if not hasattr(root, id):
                    file = open(osj(standards_home,filestr)).read()
                    addPG(id, title=title, text=file)
                    t[id]="Page Template"                    
            elif filestr[-3:] == '.py':
                id, title = cookIdAndTitle(filestr.replace('.py',''))
                if hasattr(root, id) and remove_oldstuff:
                    root.manage_delObjects([id])
                    
                if not hasattr(root, id):
                    file = open(osj(standards_home, filestr)).read()
                    id = root._setObject(id, PythonScript(id))
                    root._getOb(id).write(file)
                    t[id]="Script (Python)"

        if DestinationURL:
            msg = ''
            for k,v in t.items():
                msg = "%s (%s)\n%s"%(k,v,msg)
            
            url = AddParam2URL(DestinationURL,{'manage_tabs_message':\
                                "Standard objects deployed\n\n%s"%msg})
            self.REQUEST.RESPONSE.redirect(url)
        else:
            return "Standard objects deployed\n%s"%t

    def _deployImages(self, destination, dir, extensions=['.gif','.ico'], t={},
                      remove_oldstuff=0):
        """ do the actual deployment of images in a dir """
        # shortcuts
        osj = os.path.join
        
        for filestr in os.listdir(dir):
            if self._file_has_extensions(filestr, extensions):
                # take the image
                id, title = cookIdAndTitle(filestr)
                base= getattr(destination,'aq_base',destination)
                if hasattr(base, id) and remove_oldstuff:
                    destination.manage_delObjects([id])
                    
                if not hasattr(base, id):
                    destination.manage_addImage(id, title=title, \
                          file=open(osj(dir, filestr),'rb').read())
                    t[id]="Image"
        return t
    
    def _file_has_extensions(self, filestr, extensions):
        """ check if a filestr has any of the give extensions """
        for extension in extensions:
            if filestr.find(extension) > -1:
                return true
        return false
    
    def getHeader(self):
        """ return "this" header or inherited standard_html_header """
        if self.use_native_header:
            m = self.standard_html_header
        else:
            m = self.mass_standard_html_header
        return m(self, self.REQUEST)
    

    def getFooter(self):
        """ return "this" footer or inherited standard_html_footer """
        if self.use_native_header:
            m = self.standard_html_footer
        else:
            m = self.mass_standard_html_footer
        return m(self, self.REQUEST)
        
    def getAllRecentIssues(self):
        """ find all issues in all issuetrackers """
        allissues = []
        for issuetracker in self._interest_objectValues('Issue Tracker'):
            allissues = allissues + issuetracker.ListIssuesFiltered()
        
        datesorted = self._datesort(allissues)
        datesorted.reverse()
        
        return datesorted
        
    def _datesort(self, issues):
        """ sort all issues by date """
        return sequence.sort(issues, (('bobobase_modification_time',),))
    
    def _interest_objectValues(self, meta_type):
        """ return all the ones interested in """
        ts = []
        ignore = self.toList(self.getIgnoreTrackers())
        for o in self.objectValues(meta_type):
            if str(o.id) not in ignore:
                ts.append(o)
        
        return ts
    
    def getIgnoreTrackers(self):
        """ return what's in the request """
        request = self.REQUEST
        key = IGNORE_COOKIEKEY
        return request.get(key, [])
    
    def toList(self, stringlist):
        """ 'stringlist' looks like this issuetracker1|tracker2|tracker3 """
        if type(stringlist) == type([]):
            return stringlist
        else:
            return stringlist.split('|')
        
        
    def saveTrackersOfInterest(self, issuetrackers_show):
        """ save cookie """
        request = self.REQUEST
        response = request.RESPONSE
        
        all_issuetrackers = []
        for o in self.objectValues('Issue Tracker'):
            all_issuetrackers.append(str(o.id))
        
        key = IGNORE_COOKIEKEY
        to_save = []
        
        for id in all_issuetrackers:
            if id not in issuetrackers_show:
                to_save.append(id)
        to_save = '|'.join(to_save)
        then = DateTime()
        then = then+360
        then = then.rfc822()
        
        response.setCookie(key, to_save, path='/', expires=then)
        
        rand_str = str(int(DateTime()))
        response.redirect(self.absolute_url()+'?random=%s'%rand_str, lock=1)


    # some templates
    security.declareProtected('View', 'AllrecentIssues')
    AllrecentIssues = DTMLFile('dtml/AllrecentIssues', globals())
    trackersOfInterestForm = DTMLFile('dtml/trackersOfInterestForm', globals())

    security.declareProtected('View', 'index_html')
    index_html = DTMLFile('dtml/index_html', globals())
    mass_standard_html_header = DTMLFile('dtml/standard_html_header', globals())
    mass_standard_html_footer = DTMLFile('dtml/standard_html_footer', globals())
    masscontainer_style_css = DTMLFile('dtml/masscontainer_style.css', globals())

    def RSS091(self, batchsize=None, withheaders=1):
        """ return RSS XML """
        request = self.REQUEST
        root = self.getRoot()
        header="""<?xml version="1.0"?><rss version="0.91">
        <channel>
        <title>%s</title>
        <link>%s</link>
        <description>%s</description>
        <language>en-uk</language>
        <copyright></copyright>
        <webMaster></webMaster>\n"""%\
           (root.title, root.absolute_url(), root.title)
        #logo = getattr(self, 'issuetracker_logo.gif')
        #header=header+"""<image>
        #<title>%s</title>
        #<url>%s</url>
        #<link>%s</link>
        #<width>%s</width>
        #<height>%s</height>
        #<description>%s</description>
        #</image>\n"""%(logo.title, logo.absolute_url().strip(),
                       #root.absolute_url(),
                       #logo.width, logo.height,
                       #root.title)
        # manually set sortorder
        request.set('sortorder','date')
        request.set('reverse',1)
        xml=''
        if batchsize is None:
            batchsize = 10
        for issue in self.getAllRecentIssues()[:batchsize]:
            title = "%s (%s)"%(issue.title, issue.status.capitalize()) 
            title = self._prepare_feed(title)
            description = self._prepare_feed(issue.description)

            xml=xml+"""\n\t<item>
            <title>%s</title>
            <description>%s</description>
            <link>%s</link>
            """%(title, description, issue.absolute_url())
            if issue.fromname != '':
                author = "%s (%s)"%(issue.fromname, issue.email)
                xml="%s\n<author>%s</author>\n"%(xml, author)
            xml=xml+"\n\t</item>"
            
        footer="""</channel></rss>>"""
        if withheaders:
            xml = header+xml+footer
            
        response = request.RESPONSE
        response.setHeader('Content-Type', 'text/xml')
        return xml
    
    def _prepare_feed(self, s):
        """ prepare the text for XML usage """
        _replace = _replace_special_chars
        s = html_quote(s)
        s = s.replace('','&#163;')
        if _replace is not None:
            s = _replace(s, html_encoding=1)
        s = s.replace('&','&amp;')
        return s
    
MassContainer.__dict__['masscontainer_style.css'] = MassContainer.masscontainer_style_css
MassContainer.__dict__['rss.xml'] = MassContainer.RSS091

    
